'''
Demonstrates a small memory. Pushing the "Go"
button reads or writes memory depending on the setting of
the Combobox.
'''
import tkinter
from tkinter import ttk
import jtk
import random

MEM_SIZE = 32
VAL_SIZE = 1024

class Memory(jtk.McListBox):
    def __init__(self, parent, headers, size):
        super().__init__(parent, headers, selectmode="none")
        self.size = size
        for addr in range(size):
            self.tree.insert('', 'end', self.idFromIndex(addr),
                     values=(addr, random.randint(0, VAL_SIZE-1)))
        self.tree["height"] = size

    def read(self, addr):
        return self.getByIndex(addr)[1]

    def write(self, addr, value):
        self.setByIndex(addr, 1, value)  ## value goes in column 1

class App(ttk.Frame):
    def __init__(self, parent):
        super().__init__(parent)
        self.win = win = tkinter.Canvas(self, width=400, height=700)
        win.pack(side="left")
        self.mem = Memory(self, ("address", "contents"), MEM_SIZE)
        self.mem.pack(side="left")

        self.ab = ab = jtk.RTVEntry(self, self.addressOK, self.doBad, width=25, justify="center")
        ab.set(random.randint(0, MEM_SIZE-1))
        jtk.TextLine.ptLen(win, (400, 200), 300, 50, "e", ab, 40)
        win.create_text(250, 160, text="address bus", anchor='c')
        
        self.rw = rw = ttk.Combobox(self, values=["read", "write"], width=5,
                                    state='readonly', takefocus=False)
        rw.set("read")
        jtk.TextLine.ptLen(win, (400, 320), 300, 1, 'e', rw, 60)

        gb = ttk.Button(self, text="Go", width=5, command=self.readWrite)
        jtk.TextLine.ptLen(win, (400, 380), 300, 1, 'e', gb, 60)

        self.db = db = jtk.RTVEntry(self, self.dataOK, self.doBad, width=25, justify="center")
        db.set(random.randint(0, VAL_SIZE-1))
        jtk.TextLine.ptLen(win, (400, 500), 300, 50, "e", db, 40)
        win.create_text(250, 460, text="data bus", anchor='c')

    def readWrite(self):
        readOrWrite = self.rw.get()
        if readOrWrite == "read":
            addr = self.ab.get()
            addr = int(addr)
            val = self.mem.read(addr)
            self.db.set(val)
        elif readOrWrite == 'write':
            addr = self.ab.get()
            addr = int(addr)
            val = self.db.get()
            self.mem.write(addr, val)
        
    def doBad(self):
        self.bell()
            

    def addressOK(self, string, dummy):
        return self.resultOK(string, dummy, MEM_SIZE-1)

    def dataOK(self, string, dummy):
        return self.resultOK(string, dummy, VAL_SIZE-1)
   
    def resultOK(self, string, dummy, upperLimit):
        if string == '':
            retval = True, '0', 1                 ## third value positions cursor
        else:
            try:
                result = int(string)
            except:
                return False, None                ## no change, so bail out now
            else:
                retval = 0 <= result <= upperLimit, self.trim(string)
        return retval

    def trim(self, string):  ## get rid of leading zeroes but not entire string
            i = 0
            for i in range(len(string) - 1):
                if string[i] != '0': break
                i += 1
            return string[i:]
    
root = tkinter.Tk()
root.title("Model Memory")
app = App(root)
app.pack()
root.mainloop()
